"""
This script shows how to download a shielded app
You must use the UUID obtained with the start_shielding_script.py as inout for the 'shieldingUUID'
- shieldingUUID: The identifier of the shielding process
In this script you can see how the endpoints are used in order to:
    1. Obtain authorization
    2. Obtain pre-signed URL to download the shielded app
    And how to download the shielding app by using the pre-signed URL
"""
import requests
import os
import argparse
from requests.auth import HTTPBasicAuth
from urllib.parse import urlparse
from dotenv import load_dotenv


load_dotenv(override=False)


def get_env_var(env_var_name: str):
    env_var_value = os.getenv(env_var_name, '')
    if not env_var_value.strip():
        raise Exception(f"Missing environment variable required: {env_var_name}")
    return env_var_value


client_id = get_env_var('SHIELDINGAPPS_CLIENT_ID')
client_secret = get_env_var('SHIELDINGAPPS_CLIENT_SECRET')
auth_server_url = get_env_var('SHIELDINGAPPS_AUTH_SERVER_URL')
print(f"Variable Environment [SHIELDINGAPPS_AUTH_SERVER_URL]={auth_server_url}")
base_shieldingapps_api_url = get_env_var('SHIELDINGAPPS_BASE_SHIELDINGAPPS_API_URL')
print(f"Variable Environment [SHIELDINGAPPS_BASE_SHIELDINGAPPS_API_URL]={base_shieldingapps_api_url}")


def download_shielded_app(shielding_uuid: str):
    print("### Starting Download Shielding App...")

    payload = 'grant_type=client_credentials'
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
    response = requests.request("POST", auth_server_url, headers=headers, data=payload,
                                auth=HTTPBasicAuth(client_id, client_secret))
    if not response.status_code == 200:
        print(f"Response-status_code: {response.status_code} - [{response.text}]")
        raise Exception("Error when trying to get authorization!!!")

    access_token = response.json().get('access_token')

    # 9. After shielding is completed, shielded mobile App Download URL can be obtained via REST API (Optional step)
    download_shieldingapps_api_url = f"{base_shieldingapps_api_url}?uuid={shielding_uuid}&action=download"
    headers = {'Authorization': f"Bearer {access_token}"}
    response_download = requests.request("GET", download_shieldingapps_api_url, headers=headers)

    if response_download.status_code == 200:
        response_download_url = response_download.json().get('url')
    else:
        print(f"Response-status_code: {response_download.status_code} - [{response_download.text}]")
        raise Exception("Something went wrong when trying to get App shielding download URL!!")

    # 10. Download shielded mobile app via Download URL received in previous step (Optional step)
    shielded_app_download_url = response_download_url
    response_shielded_download = requests.request("GET", shielded_app_download_url)

    if response_shielded_download.status_code == 200:
        print(f"File successfully downloaded")
        shielded_filename = os.path.basename(urlparse(response_shielded_download.url).path)
        output_shielded_file = os.path.join(shielded_filename)  # Stored locally. Update it to your output path
        print(f"Filename: {shielded_filename}")
        with open(output_shielded_file, "wb") as file:
            file.write(response_shielded_download.content)
        print(f"File successfully stored")
    else:
        print(f"Response-status_code: {response_shielded_download.status_code} - [{response_shielded_download.text}]")
        raise Exception("Something went wrong when trying to download the shielded App!!")

    print("### Download Shielding App Finished!")



def main():
    parser = argparse.ArgumentParser(description="Download shielded mobile application.")
    parser.add_argument('--shieldingUUID', type=str, required=True,
                        help="The UUID reference of the shielding process.")
    args = parser.parse_args()
    download_shielded_app(args.shieldingUUID)


if __name__ == "__main__":
    main()
