"""
This script shows how to start shielding a mobile application
It requires some parameters:
- configurationKey: Obtained from the Shielding Portal UI
- shielderPlatform: ANDROID or IOS
- fileNameWithExt: Name of the mobile app to be shielded including its extension.
You must use the UUID outcome of the process as input for the other scripts
which is the identifier of the shielding process
In this script you can see how the endpoints are used in order to:
    1. Obtain authorization
    2. Obtain the pre-signed URL and start the shielding
    3. Start the shielding
"""
import argparse
import json
import os
import requests
from requests.auth import HTTPBasicAuth
from dotenv import load_dotenv


load_dotenv(override=False)


def get_env_var(env_var_name: str):
    env_var_value = os.getenv(env_var_name, '')
    print(env_var_name, env_var_value)
    if not env_var_value.strip():
        raise Exception(f"Missing environment variable required: {env_var_name}")
    return env_var_value


client_id = get_env_var('SHIELDINGAPPS_CLIENT_ID')
client_secret = get_env_var('SHIELDINGAPPS_CLIENT_SECRET')
auth_server_url = get_env_var('SHIELDINGAPPS_AUTH_SERVER_URL')
print(f"Variable Environment [SHIELDINGAPPS_AUTH_SERVER_URL]={auth_server_url}")
base_shieldingapps_api_url = get_env_var('SHIELDINGAPPS_BASE_SHIELDINGAPPS_API_URL')
print(f"Variable Environment [SHIELDINGAPPS_BASE_SHIELDINGAPPS_API_URL]={base_shieldingapps_api_url}")


def start_shielding_app(configuration_key: str, shielder_platform: str, file_name_with_ext: str):
    print("### Starting Shielding App...")
    # 4. Using Client ID/Secret, obtain the Access Token via Auth Server
    payload = 'grant_type=client_credentials'
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
    response = requests.request("POST", auth_server_url, headers=headers, data=payload,
                                auth=HTTPBasicAuth(client_id, client_secret))

    if not response.status_code == 200:
        print(f"Response-status_code: {response.status_code} - [{response.text}]")
        raise Exception("Error when trying to get authorization!!!")

    access_token = response.json().get('access_token')

    # 5. GET Upload URL(To Upload Mobile App to Storage Server) via REST API
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f"Bearer {access_token}"
    }
    payload = json.dumps({
        "configurationKey": configuration_key,
        "shielderPlatform": shielder_platform,
        "fileNameWithExt": file_name_with_ext
    })
    response = requests.request("POST", base_shieldingapps_api_url, headers=headers, data=payload)

    if response.status_code == 201:
        print(f"Upload URL successfully created")
        pre_signed_url = response.json().get('url')  # The URL for uploading the mobile app to be shielded.
        shielding_uuid = response.json().get('uuid')  # The UUID reference to the shielding process.
    else:
        print(f"Response-status_code: {response.status_code} - [{response.text}]")
        raise Exception("Something went wrong when trying to obtained Upload URL!!")

    # 6. Upload Mobile App to Storage Server via Upload URL received in previous step
    file_path = file_name_with_ext  # Replace with the path to your local file e.g: /home/apps/app.apk
    try:
        with open(file_path, 'rb') as file:
            payload_file_content = file.read()
    except FileNotFoundError:
        raise Exception(f"File not found: {file_path}")

    headers = {'Content-Type': 'application/octet-stream'}
    response = requests.request("PUT", pre_signed_url, headers=headers, data=payload_file_content)
    response.raise_for_status()
    print("File uploaded successfully!!")

    # 7. After Upload to storage is complete, invoke Start Shielding REST API
    start_shieldingapps_api_url = f"{base_shieldingapps_api_url}?uuid={shielding_uuid}"
    payload = json.dumps({
        "configurationKey": configuration_key,
        "fileNameWithExt": file_name_with_ext
    })
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f"Bearer {access_token}"  # Access token obtained in step 4.
    }
    response = requests.request("POST", start_shieldingapps_api_url, headers=headers, data=payload)

    if response.status_code == 201:
        print(f"Application shielding successfully started [SHIELDING_UUID]={response.json().get('uuid')}")
    else:
        print(f"Response-status_code: {response.status_code} - [{response.text}]")
        raise Exception("Something went wrong when trying to start App shielding!!")

    print("### Starting Shielding App Finished!")
    return response.json().get('uuid')


def main():
    parser = argparse.ArgumentParser(description="Start Shielding a mobile application.")
    parser.add_argument('--configurationKey', type=str, required=True,
                        help="The Configuration Key associated to the Project/Config for shielding the application.")
    parser.add_argument('--shielderPlatform', type=str, required=True,
                        help="The Platform (ANDROID or IOS) of the mobile application.")
    parser.add_argument('--fileNameWithExt', type=str, required=True,
                        help="The mobile application file name with extension")
    args = parser.parse_args()
    start_shielding_app(args.configurationKey, args.shielderPlatform, args.fileNameWithExt)


if __name__ == "__main__":
    main()
